Package: KinSwingR
Type: Package
Title: KinSwingR: network-based kinase activity prediction
Version: 1.28.0
Authors@R: person("Ashley J.", "Waardenberg", email =
    "a.waardenberg@gmail.com", role=c("aut", "cre"))
Description: KinSwingR integrates phosphosite data derived from mass-spectrometry
    data and kinase-substrate predictions to predict kinase activity. Several functions
    allow the user to build PWM models of kinase-subtrates, statistically infer PWM:substrate
    matches, and integrate these data to infer kinase activity.
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5)
Imports: data.table, BiocParallel, sqldf, stats, grid, grDevices
biocViews: Proteomics, SequenceMatching, Network
RoxygenNote: 6.1.0
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/KinSwingR
git_branch: RELEASE_3_22
git_last_commit: 5de1986
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:42:26 UTC; biocbuild
Author: Ashley J. Waardenberg [aut, cre]
Maintainer: Ashley J. Waardenberg <a.waardenberg@gmail.com>
