# Generated by roxygen2: do not edit by hand

export(buildPWM)
export(cleanAnnotation)
export(scoreSequences)
export(swing)
export(viewPWM)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(data.table,data.table)
importFrom(data.table,melt.data.table)
importFrom(grDevices,rgb)
importFrom(grid,dataViewport)
importFrom(grid,gpar)
importFrom(grid,grid.newpage)
importFrom(grid,grid.polygon)
importFrom(grid,grid.xaxis)
importFrom(grid,grid.yaxis)
importFrom(grid,plotViewport)
importFrom(grid,pushViewport)
importFrom(sqldf,sqldf)
importFrom(stats,sd)
importFrom(stats,setNames)
