# KinSwingR

KinSwingR is an R package for inference of kinase activity from phosphoproteomics data.

## Installing KinSwingR

To obtain the latest version from github, install devtools in R and use the following:

```R
library(devtools)
devtools::install_github("awaardenberg/KinSwingR", build_vignettes = TRUE)
```

## Examples

To run KinSwingR, load the library and follow the examples in the vignette.

```R
library(KinSwingR)
vignette("KinSwingR")
```

## Contact

For more details, contact Ashley Waardenberg:
a.waardenberg@gmail.com
