#' Example phosphoproteome.
#'
#' A dataset containing annotated subtrate sequences derived
#' from XXX. See original publication for more
#' details: Engholm-Keller & Waardenberg AJ et al.
#'
#' @format A data frame with 6215 rows and 4 variables:
#' \describe{
#'   \item{annotation}{Annotation of phosphorylated peptides}
#'   \item{peptide}{blank - peptides need to be extracted from
#'   annotation}
#'   \item{fc}{Fold Change (log2)}
#'   \item{pval}{P-value for fold-change.}
#' }
"example_phosphoproteome"
